@@ -1,5 +1,5 @@
 package Inline::C;
-$Inline::C::VERSION = '0.51';
+$Inline::C::VERSION = '0.54';
 $Inline::C::VERSION = eval $Inline::C::VERSION;
 
 use strict;
@@ -45,7 +45,7 @@ sub validate {
     $o->{ILSM}{MAKEFILE} ||= {};
     if (not $o->UNTAINT) {
 	require FindBin;
-	$o->{ILSM}{MAKEFILE}{INC} = "-I\"$FindBin::Bin\"";
+	$o->{ILSM}{MAKEFILE}{INC} = "-I\"$FindBin::Bin\"" if not defined $o->{ILSM}{MAKEFILE}{INC};
     }
     $o->{ILSM}{AUTOWRAP} = 0 if not defined $o->{ILSM}{AUTOWRAP};
     $o->{ILSM}{XSMODE} = 0 if not defined $o->{ILSM}{XSMODE};
@@ -191,6 +191,18 @@ END
 	    $o->{STRUCT}{'.any'} = 1;
 	    next;
 	}
+        if($key eq 'PROTOTYPES') {
+          $o->{CONFIG}{PROTOTYPES} = $value;
+          next if $value eq 'ENABLE';
+          next if $value eq 'DISABLE';
+          die "PROTOTYPES can be only either 'ENABLE' or 'DISABLE' - not $value";
+        }
+        if($key eq 'PROTOTYPE') {
+          die "PROTOTYPE configure arg must specify a hash reference"
+            unless ref($value) eq 'HASH';
+          $o->{CONFIG}{PROTOTYPE} = $value;
+          next;
+        }
 	my $class = ref $o; # handles subclasses correctly.
 	croak "'$key' is not a valid config option for $class\n";
     }
@@ -579,11 +591,15 @@ sub xs_bindings {
     my $prefix = (($o->{ILSM}{XS}{PREFIX}) ?
 		  "PREFIX = $o->{ILSM}{XS}{PREFIX}" :
 		  '');
+
+    my $prototypes = defined($o->{CONFIG}{PROTOTYPES}) ? $o->{CONFIG}{PROTOTYPES}
+                                                       : 'DISABLE';
+
     my $XS = <<END;
 
 MODULE = $module	PACKAGE = $pkg	$prefix
 
-PROTOTYPES: DISABLE
+PROTOTYPES: $prototypes
 
 END
 
@@ -603,11 +619,20 @@ END
 		  join(', ', @arg_names), ")\n");
 
 	for my $arg_name (@arg_names) {
-	    my $arg_type = shift @arg_types;
-	    last if $arg_type eq '...';
-	    $XS .= "\t$arg_type\t$arg_name\n";
+	  my $arg_type = shift @arg_types;
+	  last if $arg_type eq '...';
+	  $XS .= "\t$arg_type\t$arg_name\n";
 	}
 
+        my %h;
+        if (defined($o->{CONFIG}{PROTOTYPE})) {
+           %h = %{$o->{CONFIG}{PROTOTYPE}};
+        }
+
+        if(defined($h{$function})) {
+          $XS .= "  PROTOTYPE: $h{$function}\n";
+        }
+
 	my $listargs = '';
 	$listargs = pop @arg_names if (@arg_names and
 				       $arg_names[-1] eq '...');
@@ -761,10 +786,15 @@ sub compile {
     ($cwd) = $cwd =~ /(.*)/ if $o->UNTAINT;
 
     chdir $build_dir;
-    $o->call('makefile_pl', '"perl Makefile.PL"', 2);
-    $o->call('make', '"make"', 2);
-    $o->call('make_install', '"make install"', 2);
+    # Run these in an eval block, so that we get to chdir back to
+    # $cwd if there's a failure. (Ticket #81375.)
+    eval {
+      $o->call('makefile_pl', '"perl Makefile.PL"', 2);
+      $o->call('make', '"make"', 2);
+      $o->call('make_install', '"make install"', 2);
+    };
     chdir $cwd;
+    die if $@; #Die now that we've done the chdir back to $cwd. (#81375)
     $o->call('cleanup', 'Cleaning Up', 2);
 }
 
@@ -887,6 +917,9 @@ sub fix_make {
 
 sub quote_space {
 
+  # Do nothing if $ENV{NO_INSANE_DIRNAMES} is set
+  return $_[0] if $ENV{NO_INSANE_DIRNAMES};
+
   # If $_[0] contains one or more doublequote characters, assume
   # that whitespace has already been quoted as required. Hence,
   # do nothing other than immediately return $_[0] as is.
@@ -950,6 +983,7 @@ sub quote_space {
   # original argument should not have changed in any way.
 
   my $out = join '', @in;
+  $out =~ s/"\-I\s+\//"\-I\//g;
   $_[0] = $out;
 }
 
@@ -970,7 +1004,6 @@ sub _parser_test {
     }
 
     warn $! if !open(TEST_FH, '>>', "$dir/parser_id");
-
     print TEST_FH $_[0];
     warn $! if !close(TEST_FH);
 }
